/********************************************************
 * Data Visualization - 2D Country Chart                *
 *                                                      *
 * Author: Kimberly Schlesinger                         *
 * Supervisor: George Legrady                           *
 *                                                      *
 * Purpose: Visualize interactions of library patrons   *
 * with other countries in each Dewey category.         *
 ********************************************************/

// Layout variables 
float constantHorizontalMargin = 160;
float constantVerticalMargin = 100;
color bgc = color(250); // default background color

// 2D matrix table related variables
Table table;
Table countries;
Table maxes;
Table totals;
Table subtitles;
int numRows, numColumns;
int dispRows = 0; // number of rows (dewey classes x years) to display
float[][] dataMatrix;
float[][] maxMatrix;
float[][] totMatrix;
float maxValue, minValue;

// display
int dewey_code = 9;
String defaultDeweyClass = "900";
String deweyRegExp = "9..";
String deweyTitle = "History & geography";
int noYears = 11;   // number of years per dewey class
int noInClass = 10; // size of dewey class bin
float cellWidth, cellHeight;
float yearFrac = 1; // init. size-of-bar variable as fraction of column width

// text labels
int startYear = 2006;

//interaction
boolean normalisation = false;
boolean deweySwitch = false;
boolean colorSwitch = false;
boolean overGrid = false;

void setup() {
  size(1280, 720); // setup the size of the window

  // Make the screen resizable.                     
  surface.setResizable(true);

  //Put data into Processing tables from csv files
  countries = loadTable("q3_countries.csv");
  table = loadTable("q3_country_dewey_yr.csv", "header");
  maxes = loadTable("q3_cdy_maxbins.csv","header");
  totals = loadTable("q3_cdy_totcout.csv");
  subtitles = loadTable("deweyDivisions.csv");
  
  // get dimensions of main display table
  numRows = table.getRowCount(); 
  numColumns = table.getColumnCount();
  for (TableRow row : table.matchRows("9..","deweyBin")) { 
    dispRows += 1;
  }
  
  // initialize matrices for display
  dataMatrix = new float[dispRows][numColumns-2];
  maxMatrix = new float[dispRows][numColumns-2];
  int nc = numColumns-2;
  println("Rows: " + dispRows + " Columns: " + nc);
  
  // set totMatrix to 1 for no normalisation
  totMatrix = new float[1][numColumns-2];
  for (int i=0; i<numColumns-2; i++) {
    totMatrix[0][i] = 1;
  }
  
  // Put data into dataMatrix & maxMatrix...
  loadgrid();

}

void draw() {
  
  //refresh the canvas every frame
  
  //background color
  background(bgc);
  
  //modifying the cell size according to window size
  cellWidth = (width - constantHorizontalMargin*2)/(numColumns-2);
  cellHeight = (height - constantVerticalMargin*2)/dispRows;
      
  //Draw 2D Matrix in grayscale (default)
  grayscaleApply();

  //Draw vertical labels
  for(int i=0; i<numColumns-2; i++) {
    textAlign(CENTER, CENTER);
    fill(0);
    textSize(12);
    String cname = countries.getString(0,i+2);
    text(cname, constantHorizontalMargin + cellWidth/2 + cellWidth*i,
         constantVerticalMargin - cellHeight*2);
  }
  
  // Draw horizontal labels
  for(int i=0; i<noInClass; i++) {
    textAlign(RIGHT,CENTER);
    fill(0);
    textSize(12);
    text(int(defaultDeweyClass)+i*10, 
         constantHorizontalMargin - cellWidth/4,
         constantVerticalMargin + cellHeight*(noYears+1)/2 + cellHeight*(noYears+1)*i);
  }
       
  //Draw Title
  textAlign(CENTER, CENTER);
  textSize(16);
  text(defaultDeweyClass + ": " + deweyTitle, width/2, constantVerticalMargin/2);
  
  // get mouse location and set mouseover variables
  parseMouseLocation(mouseX,mouseY);
  
  // interaction switches
  
  if (deweySwitch == true) switchLevel();

  if (normalisation == true) normalise();
  else if (normalisation == false) reset(); 
  
  if (colorSwitch == true) colorApply();
  else if (colorSwitch == false) grayscaleApply();
  
  if (overGrid == true) displayInfo();
  else if (overGrid == false) rewrite();
    
}